/**
 * @Author: wzy
 * @Date: 2021-10-25
 * @LastEditTime: 
 * @LastEditors: Please set LastEditors
 * @Description: In User Settings Edit
 * @filePath:
 */

#include "PD_DynamicObjRisk.h"
using namespace ADC::PD_DynmcObjRsk;

DynamicObjRisk::DynamicObjRisk(const PD::PD_Input &pd_input,
                               const std::vector<ADC::PD_EgoVehTrj::EgoVeh_TrjPnt>& EgoVeh_trace_list,
                               const std::vector<ADC::PD_SVEstmd::PD_obj_SV>& SudVeh_trace_list){

    if (init(pd_input, EgoVeh_trace_list, SudVeh_trace_list) == false){
        // std::cout << "ATTENTION: DynamicObjRisk init is false!" << std::endl;
        return;
    }
}

bool DynamicObjRisk::init(const PD::PD_Input &pd_input,
                          const std::vector<ADC::PD_EgoVehTrj::EgoVeh_TrjPnt>& EgoVeh_trace_list,
                          const std::vector<ADC::PD_SVEstmd::PD_obj_SV>& SudVeh_trace_list){
                              
    bool Flag = true;
    Flag = Flag && !(pd_input.rm_roadmodel.empty());       //check roadmodel 
    
    if(Flag== false){
        initFlag = false;
    }
    else{
        _egoVeh_TrjPnt.clear();
        // _egoVeh_TrjPnt.reserve(100);
        _allSurdVeh_TrjPnt.clear();
        
        _lane_list.clear();
        _object_list.clear();
        initFlag = true;

        _ego_lane_id = pd_input.rm_roadmodel.at(rm_Num).car_position.ego_lane_id;
        _lane_list = pd_input.rm_roadmodel.at(rm_Num).lane_list;
        _object_list = pd_input.rm_roadmodel.at(rm_Num).object_list;
        Action_Type = pd_input.rm_roadmodel.at(rm_Num).passage_list.at(psg_Num).action_type;
        GetNeighborLaneObjID(_ego_lane_id, _lane_list);

        interest_obj_id_list = pd_input.rm_roadmodel.at(rm_Num).passage_list.at(psg_Num).interest_obj_id_list;
        if(!interest_obj_id_list.empty() && interest_obj_id_list.size() >= 4){
            _ego_front_id = interest_obj_id_list[0];
            _Target_front_id = interest_obj_id_list[1];
            _Target_back_id = interest_obj_id_list[2];
            _ego_back_id = interest_obj_id_list[3];
        }
        
        RoadID.x = pd_input.rm_roadmodel.at(rm_Num).road_id.x;
        RoadID.y = pd_input.rm_roadmodel.at(rm_Num).road_id.y;
        PassageID.x = pd_input.rm_roadmodel.at(rm_Num).passage_list.at(psg_Num).passage_id.x;
        PassageID.y = pd_input.rm_roadmodel.at(rm_Num).passage_list.at(psg_Num).passage_id.y;

        PD_DynmcObjRsk::EgoVeh_Point egoveh_TrjPnt;
        for (int i = 0; i < EgoVeh_trace_list.size(); i++){
            egoveh_TrjPnt.t = EgoVeh_trace_list[i].t;
            egoveh_TrjPnt.s = EgoVeh_trace_list[i].s;
            egoveh_TrjPnt.l = EgoVeh_trace_list[i].l;
            egoveh_TrjPnt.vs = EgoVeh_trace_list[i].vx;
            egoveh_TrjPnt.as = EgoVeh_trace_list[i].ax;
            egoveh_TrjPnt.jks = EgoVeh_trace_list[i].jkx;
            _egoVeh_TrjPnt.push_back(egoveh_TrjPnt);
        }
        // std::vector<EgoVeh_Point>(_egoVeh_TrjPnt).swap(_egoVeh_TrjPnt);
        // LOG(INFO) << _egoVeh_TrjPnt << std::endl;

        for (int i = 0; i < SudVeh_trace_list.size(); i++){
            SurdVeh_TrjPnt surdVeh_TrjPnt;
            // ID
            surdVeh_TrjPnt.obj_ID = SudVeh_trace_list[i].m_obj_ID;
            // Carinfo
            for (auto &element : pd_input.rm_roadmodel.at(rm_Num).object_list){
                if (element.object_id == SudVeh_trace_list[i].m_obj_ID){
                    surdVeh_TrjPnt.surdVeh_Class = element.classification;
                    surdVeh_TrjPnt.surdVeh_Width = element.size_width;
                    surdVeh_TrjPnt.surdVeh_Length = element.size_length;
                    break;
                }
            }
            // SV
            for (int j = 0; j < SudVeh_trace_list[i].pd_ls_point_list.size(); j++){
                SV_Point sv_Point;
                sv_Point.t = SudVeh_trace_list[i].pd_ls_point_list.at(j).t;
                sv_Point.s_rel = SudVeh_trace_list[i].pd_ls_point_list.at(j).s_relative;
                sv_Point.l = SudVeh_trace_list[i].pd_ls_point_list.at(j).l;
                sv_Point.vs = SudVeh_trace_list[i].pd_ls_point_list.at(j).vs;
                sv_Point.vl = SudVeh_trace_list[i].pd_ls_point_list.at(j).vl;
                surdVeh_TrjPnt.sv_point_list.push_back(sv_Point);
            }
            _allSurdVeh_TrjPnt.push_back(surdVeh_TrjPnt);
        }
    }
    return initFlag;
}

void DynamicObjRisk::process(){

    if (initFlag != true){
        // std::cout << "ATTENTION: Data reading of PD_DynamicObjRisk is error" << std::endl;
        return;
    }

    m_DynObjRsk_list.clear();
    if (_object_list.empty() || _egoVeh_TrjPnt.empty()){
        // std::cout << "ATTENTION: <DynamicObjRisk> object_list is empty!" << std::endl;
        DynamicRiskDefultOut();
    }
    else{
        CalculateDynamicRisk();
        DynamicRiskPostProcess(_tmpdynamicrisk);
        // CalculateDynamicRiskOrn(m_DynObjRsk_list);
    }

    //output to topic
    ObserveVal();

    #if FLAG_OUTPUT != 0
    OutPutToOfs();
    #endif
}

std::vector<DynamicRisk> DynamicObjRisk::DynamicRiskPostProcess(std::map<uint32_t, TmpDynamicRisk> &_tmpdynamicrisk){
    
    // GetNeighborLaneObjID(_ego_lane_id, _lane_list);

    if (Action_Type == 4 || Action_Type == 5){
        if(!_tmpdynamicrisk.empty()){
            std::map<uint32_t, TmpDynamicRisk>::iterator it;
            for (it = _tmpdynamicrisk.begin(); it != _tmpdynamicrisk.end(); it++){
                int tmpsize = it->second.tmp_ObjLongPreClsnTTC.size();
                int lengsize = std::min(tmpsize, PD_CostSizeObjLCP);
                DynamicRisk dynamicrisk;
                dynamicrisk.ObjID = it->second.tmp_ObjID;
                for (int i = 0; i < lengsize; i++){
                    dynamicrisk.m_ObjLongPreClsnTTC.push_back(it->second.tmp_ObjLongPreClsnTTC[i]);
                    dynamicrisk.m_ObjLongPreClsnTD.push_back(it->second.tmp_ObjLongPreClsnTD[i]);
                    dynamicrisk.m_ObjLongOvlpDeg.push_back(it->second.tmp_ObjLongOvlpDeg[i]);
                    dynamicrisk.m_ObjLatOvlpDeg.push_back(it->second.tmp_ObjLatOvlpDeg[i]);
                }
                dynamicrisk.m_ObjLongCrsSfZnItg = CalculateLongCrsSfZnItg(dynamicrisk.m_ObjLongOvlpDeg);
                dynamicrisk.m_ObjLatCrsSfZnItg = CalculateLatCrsSfZnItg(dynamicrisk.m_ObjLatOvlpDeg);
                std::vector<float> _lgdislist(it->second.tmp_ObjLongMinDis.begin(), it->second.tmp_ObjLongMinDis.begin() + lengsize);
                dynamicrisk.m_ObjLongMinDis = CalLgMinDis(_lgdislist);
                std::vector<float> _latdislist(it->second.tmp_ObjLatMinDis.begin(), it->second.tmp_ObjLatMinDis.begin() + lengsize);
                dynamicrisk.m_ObjLatMinDis = CalLatMinDis(_latdislist);
                m_DynObjRsk_list.push_back(dynamicrisk);
            }
        }
    }
    else{
        if (!ObjInEgoLaneList.empty() && !_tmpdynamicrisk.empty()){
            for (auto &ele : ObjInEgoLaneList){
                std::map<uint32_t, TmpDynamicRisk>::iterator pos = _tmpdynamicrisk.find(ele.object_id);
                if (pos != _tmpdynamicrisk.end()){
                    int tmpsize = (*pos).second.tmp_ObjLongPreClsnTTC.size();
                    int lengsize = std::min(tmpsize, PD_CostSizeObjInEgoLaneList);
                    PushFun(pos, lengsize);
                }
            }
        }
        if (!ObjInEgoLaneListL.empty() && !_tmpdynamicrisk.empty()){
            for (auto &ele : ObjInEgoLaneListL){
                std::map<uint32_t, TmpDynamicRisk>::iterator pos = _tmpdynamicrisk.find(ele.object_id);
                if (pos != _tmpdynamicrisk.end()){
                    int tmpsize = (*pos).second.tmp_ObjLongPreClsnTTC.size();
                    int lengsize = std::min(tmpsize, PD_CostSizeObjInEgoLaneListL);
                    PushFun(pos, lengsize);
                }
            }
        }
        if (!ObjInEgoLaneListR.empty() && !_tmpdynamicrisk.empty()){
            for (auto &ele : ObjInEgoLaneListR){
                std::map<uint32_t, TmpDynamicRisk>::iterator pos = _tmpdynamicrisk.find(ele.object_id);
                if (pos != _tmpdynamicrisk.end()){
                    int tmpsize = (*pos).second.tmp_ObjLongPreClsnTTC.size();
                    int lengsize = std::min(tmpsize, PD_CostSizeObjInEgoLaneListL);
                    PushFun(pos, lengsize);
                }
            }
        }
        if (!ObjInEgoLaneListLL.empty() && !_tmpdynamicrisk.empty()){
            for (auto &ele : ObjInEgoLaneListLL){
                std::map<uint32_t, TmpDynamicRisk>::iterator pos = _tmpdynamicrisk.find(ele.object_id);
                if (pos != _tmpdynamicrisk.end()){
                    int tmpsize = (*pos).second.tmp_ObjLongPreClsnTTC.size();
                    int lengsize = std::min(tmpsize, PD_CostSizeObjInEgoLaneListLL);
                    PushFun(pos, lengsize);
                }
            }
        }
        if (!ObjInEgoLaneListRR.empty() && !_tmpdynamicrisk.empty()){
            for (auto &ele : ObjInEgoLaneListRR){
                std::map<uint32_t, TmpDynamicRisk>::iterator pos = _tmpdynamicrisk.find(ele.object_id);
                if (pos != _tmpdynamicrisk.end()){
                    int tmpsize = (*pos).second.tmp_ObjLongPreClsnTTC.size();
                    int lengsize = std::min(tmpsize, PD_CostSizeObjInEgoLaneListLL);
                    PushFun(pos, lengsize);
                }
            }
        }
        if(!_tmpdynamicrisk.empty()){
            std::map<uint32_t, TmpDynamicRisk>::iterator it;
            for (it = _tmpdynamicrisk.begin(); it != _tmpdynamicrisk.end(); it++){
                int tmpsize = it->second.tmp_ObjLongPreClsnTTC.size();
                int lengsize = std::min(tmpsize, PD_CostSizeObjOtherLaneList);
                DynamicRisk dynamicrisk;
                dynamicrisk.ObjID = it->second.tmp_ObjID;
                for (int i = 0; i < lengsize; i++){
                    dynamicrisk.m_ObjLongPreClsnTTC.push_back(it->second.tmp_ObjLongPreClsnTTC[i]);
                    dynamicrisk.m_ObjLongPreClsnTD.push_back(it->second.tmp_ObjLongPreClsnTD[i]);
                    dynamicrisk.m_ObjLongOvlpDeg.push_back(it->second.tmp_ObjLongOvlpDeg[i]);
                    dynamicrisk.m_ObjLatOvlpDeg.push_back(it->second.tmp_ObjLatOvlpDeg[i]);
                }
                dynamicrisk.m_ObjLongCrsSfZnItg = CalculateLongCrsSfZnItg(dynamicrisk.m_ObjLongOvlpDeg);
                dynamicrisk.m_ObjLatCrsSfZnItg = CalculateLatCrsSfZnItg(dynamicrisk.m_ObjLatOvlpDeg);
                std::vector<float> _lgdislist(it->second.tmp_ObjLongMinDis.begin(), it->second.tmp_ObjLongMinDis.begin() + lengsize);
                dynamicrisk.m_ObjLongMinDis = CalLgMinDis(_lgdislist);
                std::vector<float> _latdislist(it->second.tmp_ObjLatMinDis.begin(), it->second.tmp_ObjLatMinDis.begin() + lengsize);
                dynamicrisk.m_ObjLatMinDis = CalLatMinDis(_latdislist);
                m_DynObjRsk_list.push_back(dynamicrisk);
            }
        }
    }
    return m_DynObjRsk_list;
}

void DynamicObjRisk::GetNeighborLaneObjID(const uint8_t &_ego_lane_id,
                                          const std::vector<Lane> &_lane_id){
    
    ObjInEgoLaneList.clear();
    ObjInEgoLaneListL.clear();
    ObjInEgoLaneListLL.clear();
    ObjInEgoLaneListR.clear();
    ObjInEgoLaneListRR.clear();

    uint8_t left_neighbor_lane_id = 0;
    uint8_t right_neighbor_lane_id = 0;
    uint8_t lleft_neighbor_lane_id = 0;
    uint8_t rright_neighbor_lane_id = 0;

    if (!_lane_list.empty()){
        for (auto &ele : _lane_list){
            if ((_ego_lane_id != 0) && (ele.start_id == _ego_lane_id)){
                if (!ele.left_neighbor_lane.empty()){
                    left_neighbor_lane_id = ele.left_neighbor_lane.at(0).lane_id;
                }
                if (!ele.right_neighbor_lane.empty()){
                    right_neighbor_lane_id = ele.right_neighbor_lane.at(0).lane_id;
                }
            }
        }
        for (auto &ele : _lane_list){
            if ((left_neighbor_lane_id != 0) && (ele.start_id == left_neighbor_lane_id)){
                if (!ele.left_neighbor_lane.empty()){
                    lleft_neighbor_lane_id = ele.left_neighbor_lane.at(0).lane_id;
                }
            }
        }
        for (auto &ele : _lane_list){
            if ((right_neighbor_lane_id != 0) && (ele.start_id == right_neighbor_lane_id)){
                if (!ele.right_neighbor_lane.empty()){
                    rright_neighbor_lane_id = ele.right_neighbor_lane.at(0).lane_id;
                }
            }
        }
        for(auto &ele : _lane_list){
            if ((_ego_lane_id != 0) && (ele.start_id == _ego_lane_id)){
                for (auto &ele1 : ele.corridor_list){
                    if(int(ele1.feature_tag) == 1){
                        ObjInEgoLaneList = ele1.object_in_corridor_list;
                    }
                }
            }
            else if ((left_neighbor_lane_id != 0) && (ele.start_id == left_neighbor_lane_id)){
                for (auto &ele1 : ele.corridor_list){
                    if (int(ele1.feature_tag) == 1){
                        ObjInEgoLaneListL = ele1.object_in_corridor_list;
                    }
                }
            }
            else if ((lleft_neighbor_lane_id != 0) && (ele.start_id == lleft_neighbor_lane_id)){
                for (auto &ele1 : ele.corridor_list){
                    if (int(ele1.feature_tag) == 1){
                        ObjInEgoLaneListLL = ele1.object_in_corridor_list;
                    }
                }
            }
            else if ((right_neighbor_lane_id != 0) && (ele.start_id == right_neighbor_lane_id)){
                for (auto &ele1 : ele.corridor_list){
                    if (int(ele1.feature_tag) == 1){
                        ObjInEgoLaneListR = ele1.object_in_corridor_list;
                    }
                }
            }
            else if ((rright_neighbor_lane_id != 0) && (ele.start_id == rright_neighbor_lane_id)){
                for (auto &ele1 : ele.corridor_list){
                    if (int(ele1.feature_tag) == 1){
                        ObjInEgoLaneListRR = ele1.object_in_corridor_list;
                    }
                }
            }
            else{
                for (auto &ele1 : ele.corridor_list){
                    if (int(ele1.feature_tag) == 1){
                        ObjListOther = ele1.object_in_corridor_list;
                    }
                }
            }
        }
    } 
}

// std::vector<DynamicRisk> DynamicObjRisk::CalculateDynamicRiskOrn(std::vector<DynamicRisk> &m_DynObjRsk_list){
//     m_DynObjRsk_list_orn.clear();
//     if (!m_DynObjRsk_list.empty()){
//         DynamicRisk _dynamicrisk;
//         for(auto &ele : m_DynObjRsk_list){
//             if (ele.ObjID == _Target_front_id){
//                 _dynamicrisk.ObjID = ele.ObjID;
//                 _dynamicrisk.m_ObjLongMinDis = (ele.m_ObjLongMinDis) / tmpGainDist;
//                 _dynamicrisk.m_ObjLatMinDis = ele.m_ObjLatMinDis;
//                 _dynamicrisk.m_ObjLongCrsSfZnItg = ele.m_ObjLongCrsSfZnItg;
//                 _dynamicrisk.m_ObjLatCrsSfZnItg = ele.m_ObjLatCrsSfZnItg;
//                 for(int i = 0; i < ele.m_ObjLongPreClsnTTC.size(); i++){
//                     _dynamicrisk.m_ObjLongPreClsnTTC.push_back(ele.m_ObjLongPreClsnTTC[i]);
//                     _dynamicrisk.m_ObjLongPreClsnTD.push_back(ele.m_ObjLongPreClsnTD[i] / tmpGainTD);
//                     _dynamicrisk.m_ObjLongOvlpDeg.push_back(ele.m_ObjLongOvlpDeg[i]);
//                     _dynamicrisk.m_ObjLatOvlpDeg.push_back(ele.m_ObjLatOvlpDeg[i]);
//                 }
//                 m_DynObjRsk_list_orn.push_back(_dynamicrisk);
//             }
//             else{
//                 _dynamicrisk.ObjID = ele.ObjID;
//                 _dynamicrisk.m_ObjLongMinDis = ele.m_ObjLongMinDis;
//                 _dynamicrisk.m_ObjLatMinDis = ele.m_ObjLatMinDis;
//                 _dynamicrisk.m_ObjLongCrsSfZnItg = ele.m_ObjLongCrsSfZnItg;
//                 _dynamicrisk.m_ObjLatCrsSfZnItg = ele.m_ObjLatCrsSfZnItg;
//                 for(int i = 0; i < ele.m_ObjLongPreClsnTTC.size(); i++){
//                     _dynamicrisk.m_ObjLongPreClsnTTC.push_back(ele.m_ObjLongPreClsnTTC[i]);
//                     _dynamicrisk.m_ObjLongPreClsnTD.push_back(ele.m_ObjLongPreClsnTD[i]);
//                     _dynamicrisk.m_ObjLongOvlpDeg.push_back(ele.m_ObjLongOvlpDeg[i]);
//                     _dynamicrisk.m_ObjLatOvlpDeg.push_back(ele.m_ObjLatOvlpDeg[i]);
//                 }
//                 m_DynObjRsk_list_orn.push_back(_dynamicrisk);
//             }
//         }
//     }
//     return m_DynObjRsk_list_orn;
// }

float DynamicObjRisk::CalLgMinDis(std::vector<float> &LgDisList){
    float long_min_dis = 0.0f;
    if (!(LgDisList.empty())){
        std::vector<float> abs_lgdislist;
        for(auto &ele : LgDisList){
            abs_lgdislist.push_back(abs(ele));
        }
        std::vector<float>::iterator it = min_element(abs_lgdislist.begin(), abs_lgdislist.end());
        int index = distance(abs_lgdislist.begin(), it);
        long_min_dis = LgDisList[index];
        return long_min_dis;
    }
    else{
        long_min_dis = PDCOSTDEFAULTMAXPLUS;
        return long_min_dis;
    }
}

float DynamicObjRisk::CalLatMinDis(std::vector<float> &LatDisList){
    float lat_min_dis = 0.0f;
    if (!(LatDisList.empty())){
        lat_min_dis = *min_element(LatDisList.begin(), LatDisList.end());
        return lat_min_dis;
    }
    else{
        lat_min_dis = PDCOSTDEFAULTMAXPLUS;
        return lat_min_dis;
    }
}

void DynamicObjRisk::PushFun(std::map<uint32_t, TmpDynamicRisk>::iterator &pos, const int &lengsize){
    DynamicRisk dynamicrisk;
    dynamicrisk.ObjID = (*pos).second.tmp_ObjID;
    for (int i = 0; i < lengsize; i++){
        dynamicrisk.m_ObjLongPreClsnTTC.push_back((*pos).second.tmp_ObjLongPreClsnTTC[i]);
        dynamicrisk.m_ObjLongPreClsnTD.push_back((*pos).second.tmp_ObjLongPreClsnTD[i]);
        dynamicrisk.m_ObjLongOvlpDeg.push_back((*pos).second.tmp_ObjLongOvlpDeg[i]);
        dynamicrisk.m_ObjLatOvlpDeg.push_back((*pos).second.tmp_ObjLatOvlpDeg[i]);
    }
    // dynamicrisk.m_ObjLongPreClsnTTC((*pos).second.tmp_ObjLongPreClsnTTC.begin(), (*pos).second.tmp_ObjLongPreClsnTTC.begin() + lengsize);
    // dynamicrisk.m_ObjLongPreClsnTD((*pos).second.tmp_ObjLongPreClsnTD.begin(), (*pos).second.tmp_ObjLongPreClsnTD.begin() + lengsize);
    // dynamicrisk.m_ObjLongOvlpDeg((*pos).second.tmp_ObjLongOvlpDeg.begin(), (*pos).second.tmp_ObjLongOvlpDeg.begin() + lengsize);
    // dynamicrisk.m_ObjLatOvlpDeg((*pos).second.tmp_ObjLatOvlpDeg.begin(), (*pos).second.tmp_ObjLatOvlpDeg.begin() + lengsize);
    dynamicrisk.m_ObjLongCrsSfZnItg = CalculateLongCrsSfZnItg(dynamicrisk.m_ObjLongOvlpDeg);
    dynamicrisk.m_ObjLatCrsSfZnItg = CalculateLatCrsSfZnItg(dynamicrisk.m_ObjLatOvlpDeg);
    std::vector<float> _lgdislist((*pos).second.tmp_ObjLongMinDis.begin(), (*pos).second.tmp_ObjLongMinDis.begin() + lengsize);
    dynamicrisk.m_ObjLongMinDis = CalLgMinDis(_lgdislist);
    std::vector<float> _latdislist((*pos).second.tmp_ObjLatMinDis.begin(), (*pos).second.tmp_ObjLatMinDis.begin() + lengsize);
    dynamicrisk.m_ObjLatMinDis = CalLatMinDis(_latdislist);
    m_DynObjRsk_list.push_back(dynamicrisk);
    _tmpdynamicrisk.erase(pos);
}

void DynamicObjRisk::CalculateDynamicRisk(){

    _tmpdynamicrisk.clear();
    for (int i = 0; i < _allSurdVeh_TrjPnt.size(); i++){

        _sv_point_list.clear();
        _sv_point_list = _allSurdVeh_TrjPnt[i].sv_point_list;
        _surdVeh_Class = _allSurdVeh_TrjPnt[i].surdVeh_Class;
        _surdVeh_Width = _allSurdVeh_TrjPnt[i].surdVeh_Width;
        _surdVeh_Length = _allSurdVeh_TrjPnt[i].surdVeh_Length;

        bool sv_point_lat_flag = ObjctOpt();
        TmpDynamicRisk tmpdynamicRisk;
        if(sv_point_lat_flag){

            bool EgoBckNotLC = false;
            if(_allSurdVeh_TrjPnt[i].obj_ID == _ego_back_id){               
                if ((Action_Type == 2) && (_sv_point_list.back().l < PD_CostEgoBackLatOffSetL)){
                    EgoBckNotLC = true;
                }
                else if ((Action_Type == 3) && (_sv_point_list.back().l > PD_CostEgoBackLatOffSetR)){
                    EgoBckNotLC = true;
                }
                else if ((Action_Type == 4) || (Action_Type == 5)){
                    if (abs(_sv_point_list.back().l) < PD_CostEgoBackLatOffSetLF){
                        EgoBckNotLC = true;
                    }
                }
            }

            bool EgoLaneBckNotLC = false;
            if (!ObjInEgoLaneList.empty()){
                for (auto &ele : ObjInEgoLaneList){
                    if ((ele.object_id == _allSurdVeh_TrjPnt[i].obj_ID) && (_allSurdVeh_TrjPnt[i].obj_ID != _ego_back_id)){
                        if (_sv_point_list.front().s_rel < 0){
                            if ((Action_Type == 2) && (_sv_point_list.back().l < PD_CostEgoBackLatOffSetL)){
                                EgoLaneBckNotLC = true;
                            }
                            else if ((Action_Type == 3) && (_sv_point_list.back().l > PD_CostEgoBackLatOffSetR)){
                                EgoLaneBckNotLC = true;
                            }
                            else if ((Action_Type == 4) || (Action_Type == 5)){
                                if (abs(_sv_point_list.back().l) < PD_CostEgoBackLatOffSetLF){
                                    EgoLaneBckNotLC = true;
                                }
                            }
                        }
                    }
                }
            }

            bool OptObjLCP = false;
            if (Action_Type == 4 || Action_Type == 5){
                if (!ObjInEgoLaneList.empty()){
                    for (auto &ele : ObjInEgoLaneList){
                        if (ele.object_id == _allSurdVeh_TrjPnt[i].obj_ID){
                            OptObjLCP = false;
                            break;
                        }
                        else{
                            OptObjLCP = true;
                        }
                    }
                }
            }

            bool EgoBckOrFrt2R = false;
            if ((Action_Type == 2) && (!ObjInEgoLaneListR.empty())){
                for (auto &ele : ObjInEgoLaneListR){
                    if (ele.object_id == _allSurdVeh_TrjPnt[i].obj_ID){
                        int tmpsize = _sv_point_list.size();
                        int lengsize = std::min(tmpsize, PD_CostSizeObjInEgoLaneListL);
                        if (_sv_point_list.front().s_rel < PD_CostBckDist2R || _sv_point_list.front().s_rel > PD_CostFrtDist2R){
                            EgoBckOrFrt2R = true;
                        }
                        else{
                            if ((abs(_sv_point_list.front().l - _sv_point_list[lengsize].l) < PD_CostNearLatOffSet2R) || (_sv_point_list.front().l > _sv_point_list[lengsize].l)){
                                EgoBckOrFrt2R = true;
                            }
                        }
                        break;
                    }
                }
            }  

            bool EgoBckOrFrt2LL = false;
            if ((Action_Type == 2) && (!ObjInEgoLaneListLL.empty())){
                for (auto &ele : ObjInEgoLaneListLL){
                    if (ele.object_id == _allSurdVeh_TrjPnt[i].obj_ID){
                        if (_sv_point_list.back().l > PD_CostLatOffSetObjInLaneLL){
                            EgoBckOrFrt2LL = true;
                        }
                        break;
                    }
                }
            }         

            bool EgoBckOrFrt3L = false;
            if ((Action_Type == 3) && (!ObjInEgoLaneListL.empty())){
                for (auto &ele : ObjInEgoLaneListL){
                    if (ele.object_id == _allSurdVeh_TrjPnt[i].obj_ID){
                        int tmpsize = _sv_point_list.size();
                        int lengsize = std::min(tmpsize, PD_CostSizeObjInEgoLaneListL);
                        if (_sv_point_list.front().s_rel < PD_CostBckDist3L || _sv_point_list.front().s_rel > PD_CostFrtDist3L){
                            EgoBckOrFrt3L = true;
                        }
                        else{
                            if ((abs(_sv_point_list.front().l - _sv_point_list[lengsize].l) < PD_CostNearLatOffSet3L) || (_sv_point_list.front().l < _sv_point_list[lengsize].l)){
                                EgoBckOrFrt3L = true;
                            }
                        }
                        break;
                    }
                }
            }

            bool EgoBckOrFrt3RR = false;
            if ((Action_Type == 3) && (!ObjInEgoLaneListRR.empty())){
                for (auto &ele : ObjInEgoLaneListRR){
                    if (ele.object_id == _allSurdVeh_TrjPnt[i].obj_ID){
                        if (_sv_point_list.back().l < PD_CostLatOffSetObjInLaneRR){
                            EgoBckOrFrt3RR = true;
                        }
                        break;
                    }
                }
            } 

            bool EgoBckOrFrt2RR = false;
            if ((Action_Type == 2) && (!ObjInEgoLaneListRR.empty())){
                for (auto &ele : ObjInEgoLaneListRR){
                    if (ele.object_id == _allSurdVeh_TrjPnt[i].obj_ID){
                        if ((_sv_point_list.front().s_rel < PD_CostBckDist2RR) && (_sv_point_list.front().s_rel > PD_CostFrtDist2RR)){
                            EgoBckOrFrt2RR = true;
                        }
                        else{
                            if (_sv_point_list.front().vl < PD_CostEgoBckFrtLatVl2RR){
                                EgoBckOrFrt2RR = true;
                            }
                        }
                    }
                }
            }

            bool EgoBckOrFrt3LL = false;
            if ((Action_Type == 3) && (!ObjInEgoLaneListLL.empty())){
                for (auto &ele : ObjInEgoLaneListLL){
                    if (ele.object_id == _allSurdVeh_TrjPnt[i].obj_ID){
                        if ((_sv_point_list.front().s_rel < PD_CostBckDist3LL) && (_sv_point_list.front().s_rel > PD_CostFrtDist3LL)){
                            EgoBckOrFrt3LL = true;
                        }
                        else{
                            if (_sv_point_list.front().vl > PD_CostEgoBckFrtLatVl3LL){
                                EgoBckOrFrt3LL = true;
                            }
                        }
                    }
                }
            }
            
            if (EgoBckNotLC || EgoLaneBckNotLC || OptObjLCP || EgoBckOrFrt2R || EgoBckOrFrt2LL || EgoBckOrFrt3L || EgoBckOrFrt3RR || EgoBckOrFrt2RR || EgoBckOrFrt3LL){
                tmpdynamicRisk.tmp_ObjID = _allSurdVeh_TrjPnt[i].obj_ID;
                for (int i = 0; i < _sv_point_list.size(); i++){
                    tmpdynamicRisk.tmp_ObjLongPreClsnTTC.push_back(PDCOSTDEFAULTMAXPLUS);
                    tmpdynamicRisk.tmp_ObjLongPreClsnTD.push_back(PDCOSTDEFAULTMAXPLUS);
                    tmpdynamicRisk.tmp_ObjLongOvlpDeg.push_back(0.0f);
                    tmpdynamicRisk.tmp_ObjLatOvlpDeg.push_back(0.0f);
                    tmpdynamicRisk.tmp_ObjLongMinDis.push_back(PDCOSTDEFAULTMAXPLUS);
                    tmpdynamicRisk.tmp_ObjLatMinDis.push_back(PDCOSTDEFAULTMAXPLUS);
                }
                _tmpdynamicrisk.insert(std::pair<uint32_t, TmpDynamicRisk>(_allSurdVeh_TrjPnt[i].obj_ID, tmpdynamicRisk));
            }
            else{

                CalculateLatThreshold();
                ThresholdBoundary();

                //ID
                tmpdynamicRisk.tmp_ObjID = _allSurdVeh_TrjPnt[i].obj_ID;
                //cost项
                tmpdynamicRisk.tmp_ObjLongPreClsnTTC = CalculateTTC();
                tmpdynamicRisk.tmp_ObjLongPreClsnTD = CalculateTD();
                tmpdynamicRisk.tmp_ObjLongOvlpDeg = CalculateLongOvlpDeg();
                tmpdynamicRisk.tmp_ObjLatOvlpDeg = CalculateLatOvlpDeg();
                tmpdynamicRisk.tmp_ObjLongMinDis = CalculateLongMinDis();
                tmpdynamicRisk.tmp_ObjLatMinDis = CalculateLatMinDis();
                // tmpdynamicRisk.tmp_ObjLongCrsSfZnItg = CalculateLongCrsSfZnItg();
                // tmpdynamicRisk.tmp_ObjLatCrsSfZnItg = CalculateLatCrsSfZnItg();

                if (_allSurdVeh_TrjPnt[i].obj_ID == _Target_front_id){
                    float tmpVecRel = _sv_point_list.front().vs - _egoVeh_TrjPnt.front().vs;
                    float tmpGainDist = InterPoint<float>(PD_CostRelativeVelocity, PD_CostGainTrgtFrtDist, tmpVecRel);
                    float tmpGainTD = InterPoint<float>(PD_CostRelativeVelocity, PD_CostGainTrgtFrtTD, tmpVecRel);
                    for (int i = 0; i < tmpdynamicRisk.tmp_ObjLongMinDis.size(); i++){
                        tmpdynamicRisk.tmp_ObjLongMinDis[i] = tmpdynamicRisk.tmp_ObjLongMinDis[i] * tmpGainDist;
                    }
                    for (int i = 0; i < tmpdynamicRisk.tmp_ObjLongPreClsnTD.size(); i++){
                        tmpdynamicRisk.tmp_ObjLongPreClsnTD[i] = tmpdynamicRisk.tmp_ObjLongPreClsnTD[i] * tmpGainTD;
                    }
                }

                // if (_allSurdVeh_TrjPnt[i].obj_ID == _ego_back_id){
                //     float tmpVecRel = _sv_point_list.front().vs - _egoVeh_TrjPnt.front().vs;
                //     float tmpGainTD = InterPoint<float>(PD_CostRelativeVelocityEgoBck, PD_CostGainEgoBckTD, tmpVecRel);
                //     for (int i = 0; i < tmpdynamicRisk.tmp_ObjLongPreClsnTD.size(); i++){
                //         tmpdynamicRisk.tmp_ObjLongPreClsnTD[i] = tmpdynamicRisk.tmp_ObjLongPreClsnTD[i] * tmpGainTD;
                //     }
                // }

                // if (_allSurdVeh_TrjPnt[i].obj_ID == _Target_back_id){
                //     float tmpTTC = 0.0;
                //     if (tmpdynamicRisk.tmp_ObjLongPreClsnTTC.size() >= 5){
                //         for (int i = 0; i < 5; i++){
                //             tmpTTC += tmpdynamicRisk.tmp_ObjLongPreClsnTTC[i];
                //         }
                //         tmpTTC = tmpTTC / 5;
                //     }
                //     if (tmpTTC > 3.5){
                //         for (int i = 0; i < tmpdynamicRisk.tmp_ObjLongPreClsnTD.size(); i++){
                //             tmpdynamicRisk.tmp_ObjLongPreClsnTD[i] = tmpdynamicRisk.tmp_ObjLongPreClsnTD[i] * 1.1;
                //         }
                //         for (int i = 0; i < tmpdynamicRisk.tmp_ObjLongPreClsnTTC.size(); i++){
                //             tmpdynamicRisk.tmp_ObjLongPreClsnTTC[i] = tmpdynamicRisk.tmp_ObjLongPreClsnTTC[i] * 1.3;
                //         }
                //     }
                // }

                _tmpdynamicrisk.insert(std::pair<uint32_t, TmpDynamicRisk>(_allSurdVeh_TrjPnt[i].obj_ID, tmpdynamicRisk)); 

            }
        }
    }
}

void DynamicObjRisk::DynamicRiskDefultOut(){
    DynamicRisk dynamicRisk;
    for (int i = 0; i < 1; i++){ 
        //ID
        dynamicRisk.ObjID = 0;
        //cost项
        int size_TrjPnt = 0;
        if (_egoVeh_TrjPnt.empty()){
            size_TrjPnt = 1;
        }
        else{
            size_TrjPnt = _egoVeh_TrjPnt.size();
        }
        for (int j = 0; j < size_TrjPnt; j++){
            dynamicRisk.m_ObjLongPreClsnTTC.push_back(PDCOSTDEFAULTMAXPLUS);
            dynamicRisk.m_ObjLongPreClsnTD.push_back(PDCOSTDEFAULTMAXPLUS);
            dynamicRisk.m_ObjLongOvlpDeg.push_back(0.0f);
            dynamicRisk.m_ObjLatOvlpDeg.push_back(0.0f);
        }
        dynamicRisk.m_ObjLongMinDis = PDCOSTDEFAULTMAXPLUS;
        dynamicRisk.m_ObjLatMinDis = PDCOSTDEFAULTMAXPLUS;
        dynamicRisk.m_ObjLongCrsSfZnItg = 0.0f;
        dynamicRisk.m_ObjLatCrsSfZnItg = 0.0f;
        m_DynObjRsk_list.push_back(dynamicRisk);
    }
}

bool DynamicObjRisk::ObjctOpt(){
    bool sv_point_lat_flag = true;
    std::vector<float> lat_rel_dis;
    for(int i = 0; i < _sv_point_list.size(); i++){
        lat_rel_dis.push_back(abs(_sv_point_list[i].l - _egoVeh_TrjPnt[i].l));
    }
    if(!lat_rel_dis.empty()){
        if(std::all_of(lat_rel_dis.begin(), lat_rel_dis.end(), [] (float ele) {return ele > PD_CostInterstObjLatDis;})){
            sv_point_lat_flag = false;
        }
    }
    return sv_point_lat_flag;
}

template <class T1>
T1 DynamicObjRisk::InterPoint(const std::vector<T1> &x_, const std::vector<T1> &y_, T1 &x_point){

    T1 value;
    T1 xA_point;
    T1 xB_point;
    T1 yA_point;
    T1 yB_point;

    if (!x_.empty() && !y_.empty() && x_.size() == y_.size()){
        if (x_point <= x_.front()){
            value = y_.front();
        }

        if (x_point >= x_.back()){
            value = y_.back();
        }

        if (x_point > x_.front() && x_point < x_.back()){
            for (int i = 1; i < x_.size(); i++){
                if (x_point < x_[i]){
                    xA_point = x_[i-1];
                    xB_point = x_[i];
                    yA_point = y_[i-1];
                    yB_point = y_[i];
                    break;
                }
            }
            value = (yB_point - yA_point) / (xB_point - xA_point) * (x_point - xA_point) + yA_point;
        }
        return value;
    }
    else{
        return T1(1);
    }
}

std::vector<float> DynamicObjRisk::CalculateLatThreshold(){
    // TODO
    int size_TrjPnt = 0;
    if (!(_egoVeh_TrjPnt.size() == _sv_point_list.size())){
        size_TrjPnt = std::min(_egoVeh_TrjPnt.size(), _sv_point_list.size());
    }
    else{
        size_TrjPnt = _egoVeh_TrjPnt.size();
    }
    // if (!(_egoVeh_TrjPnt.size() == _sv_point_list.size())){
    //     // std::cout << "ATTENTION: _egoVeh_TrjPnt and _sv_point_list's size is not equal!" << std::endl;
    //     std::vector<float> void_list;
    //     return void_list;
    // }
    _latthreshold_list.clear();
    float latthreshold = 0;
    for (int i = 0; i < size_TrjPnt; i++){
        //Vel
        float delta_velocity = 0;
        if (_sv_point_list[i].s_rel >= 0){
            delta_velocity = _egoVeh_TrjPnt[i].vs - _sv_point_list[i].vs;
        }
        else{
            delta_velocity = _sv_point_list[i].vs - _egoVeh_TrjPnt[i].vs;
        }
        float para_THR1 = 0.0f;
        float para_THR2 = 0.0f;
        // if(Action_Type == 2 || Action_Type == 4){
        if(Action_Type == 2){
            if(_sv_point_list[i].l < _egoVeh_TrjPnt[i].l){
                para_THR1 = PD_CostLatThrDltaVminOther;
                para_THR2 = PD_CostLatThrDltaVmaxOther;
            }
            else{
                if(abs(_egoVeh_TrjPnt[i].l) > PD_CostCrossLine){
                    para_THR1 = PD_CostLatThrDltaVminSame1 + abs(_egoVeh_TrjPnt[i].l) - VEHCALPRMVEHWIDTH / 2;   
                    para_THR2 = PD_CostLatThrDltaVmaxSame1 + abs(_egoVeh_TrjPnt[i].l) - VEHCALPRMVEHWIDTH / 2;
                }
                else{
                    para_THR1 = PD_CostLatThrDltaVminSame2 + abs(_egoVeh_TrjPnt[i].l) - VEHCALPRMVEHWIDTH / 2;   
                    para_THR2 = PD_CostLatThrDltaVmaxSame2 + abs(_egoVeh_TrjPnt[i].l) - VEHCALPRMVEHWIDTH / 2;
                }
            }
        }
        // else if(Action_Type == 3 || Action_Type == 5){
        else if(Action_Type == 3){
            if(_sv_point_list[i].l > _egoVeh_TrjPnt[i].l){
                para_THR1 = PD_CostLatThrDltaVminOther;
                para_THR2 = PD_CostLatThrDltaVmaxOther;
            }
            else{
                if(abs(_egoVeh_TrjPnt[i].l) > PD_CostCrossLine){
                    para_THR1 = PD_CostLatThrDltaVminSame1 + abs(_egoVeh_TrjPnt[i].l) - VEHCALPRMVEHWIDTH / 2;   
                    para_THR2 = PD_CostLatThrDltaVmaxSame1 + abs(_egoVeh_TrjPnt[i].l) - VEHCALPRMVEHWIDTH / 2;
                }
                else{
                    para_THR1 = PD_CostLatThrDltaVminSame2 + abs(_egoVeh_TrjPnt[i].l) - VEHCALPRMVEHWIDTH / 2;   
                    para_THR2 = PD_CostLatThrDltaVmaxSame2 + abs(_egoVeh_TrjPnt[i].l) - VEHCALPRMVEHWIDTH / 2;
                }
            }
        }
        else if(Action_Type == 4 || Action_Type == 5){
            para_THR1 = PD_CostLatThrDltaVminLCP;
            para_THR2 = PD_CostLatThrDltaVmaxLCP;
        }
        else{
            para_THR1 = PD_CostLatThrDltaVminOther;
            para_THR2 = PD_CostLatThrDltaVmaxOther;
        }
        float latthreshold_rel_vel = 0;
        float delta_V1 = PD_CostDltaVmin;        //单位：m/s，（20km/h）
        float delta_V2 = PD_CostDltaVmax;        //单位：m/s，（40km/h）
        if (delta_velocity <= delta_V1){
            latthreshold_rel_vel = para_THR1;
        }
        else if (delta_velocity > delta_V1 && delta_velocity <= delta_V2){
            latthreshold_rel_vel = (delta_velocity - delta_V1) / (delta_V2 - delta_V1) 
                                 * (para_THR2 - para_THR1) + para_THR1;
        }
        else{
            latthreshold_rel_vel = para_THR2;
        }
        //relvel
        float vel_coefficient = 0;
        float vel_coefficient1 = PD_CostLatThrRelVmin;
        float vel_coefficient2 = PD_CostLatThrRelVmax;     //41.67m/s
        float vel_MAX = VELOCITYMAX;                       //单位:m/s，（150km/h）
        if (_sv_point_list[i].s_rel >= 0){
            vel_coefficient = _egoVeh_TrjPnt[i].vs / (vel_MAX + 1e-10) * (vel_coefficient2 - vel_coefficient1) + vel_coefficient1;
        }
        else{
            vel_coefficient = _sv_point_list[i].vs / (vel_MAX + 1e-10) * (vel_coefficient2 - vel_coefficient1) + vel_coefficient1;
        }
        latthreshold = latthreshold_rel_vel * vel_coefficient;
        _latthreshold_list.push_back(latthreshold);
    }
    return _latthreshold_list;
}

std::vector<bool> DynamicObjRisk::ThresholdBoundary(){
// void DynamicObjRisk::ThresholdBoundary(){
    int size_TrjPnt = 0;
    if (!(_egoVeh_TrjPnt.size() == _sv_point_list.size())){
        size_TrjPnt = std::min(_egoVeh_TrjPnt.size(), _sv_point_list.size());
    }
    else{
        size_TrjPnt = _egoVeh_TrjPnt.size();
    } 
    _threshold_boundary_flag.clear();

    bool lat_abs_dis_flag = true;
    float lat_abs_dis = abs(_sv_point_list.back().l + _sv_point_list.front().l) / 2;
    // if(Action_Type == 2 || Action_Type == 4){
    if(Action_Type == 2){
        if(_sv_point_list.front().l < _egoVeh_TrjPnt.front().l){
            if(lat_abs_dis > PD_CostLatAbsDisOther){
                lat_abs_dis_flag = false;
            }
            else{
                lat_abs_dis_flag = true;
            }
        }
        else{
            lat_abs_dis_flag = true;
        }
    }
    // else if(Action_Type == 3 || Action_Type == 5){
    else if(Action_Type == 3){
        if(_sv_point_list.front().l > _egoVeh_TrjPnt.front().l){
            if(lat_abs_dis > PD_CostLatAbsDisOther){
                lat_abs_dis_flag = false;
            }
            else{
                lat_abs_dis_flag = true;
            }
        }
        else{
            lat_abs_dis_flag = true;
        }
    }
    else{
        lat_abs_dis_flag = true;
    }

    for(int i = 0; i < size_TrjPnt;){
        float lat_rel_dis1 = abs(_sv_point_list[i].l - _egoVeh_TrjPnt[i].l) - VEHCALPRMVEHWIDTH/2 - _surdVeh_Width/2;
        if((lat_abs_dis_flag) && (lat_rel_dis1 < _latthreshold_list[i])){
            _threshold_boundary_flag.push_back(true);
            for(int j = i+1; j < size_TrjPnt; j++){
                float lat_rel_dis2 = abs(_sv_point_list[j].l - _egoVeh_TrjPnt[j].l) - VEHCALPRMVEHWIDTH/2 - _surdVeh_Width/2;
                if(lat_rel_dis2 < _latthreshold_list[j] + 0.1){
                    _threshold_boundary_flag.push_back(true);
                }
                else{
                    i = j;
                    break;
                }
            }
            if(_threshold_boundary_flag.size() == size_TrjPnt){
                break;
            }
        }
        else{
            _threshold_boundary_flag.push_back(false);
            i++;
        }
    }
    return _threshold_boundary_flag;
}

std::vector<float> DynamicObjRisk::CalculateTTC(){
    // TODO
    int size_TrjPnt = 0;
    if (!(_egoVeh_TrjPnt.size() == _sv_point_list.size())){
        // std::cout << "ATTENTION: _egoVeh_TrjPnt and _sv_point_list's size is not equal!" << std::endl;
        size_TrjPnt = std::min(_egoVeh_TrjPnt.size(), _sv_point_list.size());
    }
    else{
        size_TrjPnt = _egoVeh_TrjPnt.size();
    }
    float TTC = 0.0f;
    std::vector<float> ttc_list;
    for (int i = 0; i < size_TrjPnt; i++){
        // if ((abs(_sv_point_list[i].l - _egoVeh_TrjPnt[i].l) - VEHCALPRMVEHWIDTH/2 - _surdVeh_Width/2) < _latthreshold_list[i]){
        if((!_threshold_boundary_flag.empty()) && (_threshold_boundary_flag[i])){
            if ((_sv_point_list[i].s_rel >= 0) && (_egoVeh_TrjPnt[i].vs > _sv_point_list[i].vs)){
                if ((_sv_point_list[i].s_rel - VEHCALPRMFORAXLEDSTRETOVEHFRNT - _surdVeh_Length/2) > 0){
                    TTC = abs(_sv_point_list[i].s_rel - VEHCALPRMFORAXLEDSTRETOVEHFRNT - _surdVeh_Length/2) 
                       / (abs(_egoVeh_TrjPnt[i].vs - _sv_point_list[i].vs) + 1e-10);
                }
                else{
                    TTC = PDCOSTCALCULATEMINPLUS;
                }
            }
            else if ((_sv_point_list[i].s_rel < 0) && (_sv_point_list[i].vs > _egoVeh_TrjPnt[i].vs)){
                if ((_sv_point_list[i].s_rel + _surdVeh_Length/2 + VEHCALPRMFORREARAXISTOREARBUMPER) < 0){
                    TTC = (_sv_point_list[i].s_rel + _surdVeh_Length/2 + VEHCALPRMFORREARAXISTOREARBUMPER) 
                        / (abs(_sv_point_list[i].vs - _egoVeh_TrjPnt[i].vs) + 1e-10);
                }
                else{
                    TTC = PDCOSTCALCULATEMINNGT;
                }
            }
            else{
                TTC = PDCOSTDEFAULTMAXPLUS;   //表示TTC无穷大
            }
        } 
        else{
            TTC = PDCOSTDEFAULTMAXPLUS;   
        }
        if ((TTC >= TTCTDTHRESHOLDMAXPLUS) || (TTC <= TTCTDTHRESHOLDMAXNGT)){
            TTC = PDCOSTDEFAULTMAXPLUS;
        }
        else if ((TTC >= 0) && (TTC <= PDCOSTDEFAULTMINPLUS)){
            TTC = PDCOSTDEFAULTMINPLUS;
        }
        else if ((TTC >= PDCOSTDEFAULTMINNGT) && (TTC < 0)){
            TTC = PDCOSTDEFAULTMINNGT;
        }
        ttc_list.push_back(TTC);
    }
    return ttc_list;
}

std::vector<float> DynamicObjRisk::CalculateTD(){
    // TODO
    int size_TrjPnt = 0;
    if (!(_egoVeh_TrjPnt.size() == _sv_point_list.size())){
        size_TrjPnt = std::min(_egoVeh_TrjPnt.size(), _sv_point_list.size());
    }
    else{
        size_TrjPnt = _egoVeh_TrjPnt.size();
    }
    float TD = 0.0f;
    std::vector<float> td_list;

    for (int i = 0; i < size_TrjPnt; i++){
        // if ((abs(_sv_point_list[i].l - _egoVeh_TrjPnt[i].l) - VEHCALPRMVEHWIDTH/2 - _surdVeh_Width/2) < _latthreshold_list[i]){
        if((!_threshold_boundary_flag.empty()) && (_threshold_boundary_flag[i])){
            if (_sv_point_list[i].s_rel >= 0){
                if ((_sv_point_list[i].s_rel - VEHCALPRMFORAXLEDSTRETOVEHFRNT - _surdVeh_Length/2) > 0){
                    TD = abs(_sv_point_list[i].s_rel - VEHCALPRMFORAXLEDSTRETOVEHFRNT - _surdVeh_Length/2) / (_egoVeh_TrjPnt[i].vs + 1e-10);
                }
                else{
                    TD = PDCOSTCALCULATEMINPLUS;
                }
            }
            else{
                if ((_sv_point_list[i].s_rel + _surdVeh_Length/2 + VEHCALPRMFORREARAXISTOREARBUMPER) < 0){
                    TD = (_sv_point_list[i].s_rel + _surdVeh_Length/2 + VEHCALPRMFORREARAXISTOREARBUMPER) / (_sv_point_list[i].vs + 1e-10);
                }
                else{
                    TD = PDCOSTCALCULATEMINNGT;
                }
            }
        }
        else{
            TD = PDCOSTDEFAULTMAXPLUS;   //表示TD无穷大
        }
        if ((TD >= TTCTDTHRESHOLDMAXPLUS) || (TD <= TTCTDTHRESHOLDMAXNGT)){
            TD = PDCOSTDEFAULTMAXPLUS;
        }
        else if ((TD >= 0) && (TD <= PDCOSTDEFAULTMINPLUS)){
            TD = PDCOSTDEFAULTMINPLUS;
        }
        else if ((TD >= PDCOSTDEFAULTMINNGT) && (TD < 0)){
            TD = PDCOSTDEFAULTMINNGT;
        } 
        td_list.push_back(TD);
    }
    return td_list;   
}

std::vector<float> DynamicObjRisk::CalculateLongOvlpDeg(){
    // TODO
    int size_TrjPnt = 0;
    if (!(_egoVeh_TrjPnt.size() == _sv_point_list.size())){
        // std::cout << "ATTENTION: _egoVeh_TrjPnt and _sv_point_list's size is not equal!" << std::endl;
        size_TrjPnt = std::min(_egoVeh_TrjPnt.size(), _sv_point_list.size());
    }
    else{
        size_TrjPnt = _egoVeh_TrjPnt.size();
    }
    std::vector<float> long_threshold_ovlp;
    for (int i = 0; i < size_TrjPnt; i++){
        float ego_V1 = 11.11;    //单位：m/s，（40km/h）
        float ego_V2 = 16.67;    //单位：m/s，（60km/h）
        //相对速度低于ego_V1时的阈值
        float para_THR1 = 10;
        //相对速度高于ego_V2时的阈值
        float para_THR2 = 15;
        float lgovlp = 0;
        float egoSpd = _egoVeh_TrjPnt[i].vs;
        if (egoSpd <= ego_V1){
            lgovlp = para_THR1;
        }
        else if (egoSpd > ego_V1 && egoSpd <= ego_V2){
            lgovlp = (egoSpd - ego_V1) / (ego_V2 - ego_V1) * (para_THR2 - para_THR1) + para_THR1;
        }
        else{
            lgovlp = para_THR2;
        }
        long_threshold_ovlp.push_back(lgovlp);
    }

    float LongOvlp = 0.0f;
    _LgOvlpFlag.clear();
    bool Flag = false;
    std::vector<float> long_ovlp_list;
    for (int i = 0; i < size_TrjPnt; i++){
        // if ((abs(_sv_point_list[i].l - _egoVeh_TrjPnt[i].l) - VEHCALPRMVEHWIDTH/2 - _surdVeh_Width/2) < _latthreshold_list[i]){
        if((!_threshold_boundary_flag.empty()) && (_threshold_boundary_flag[i])){
            if (_sv_point_list[i].s_rel >= 0){
                LongOvlp = long_threshold_ovlp[i] - (_sv_point_list[i].s_rel - VEHCALPRMFORAXLEDSTRETOVEHFRNT - _surdVeh_Length/2);
                if (LongOvlp > 0){
                    Flag = true;
                    _LgOvlpFlag.push_back(Flag);
                    long_ovlp_list.push_back(LongOvlp);
                }
                else{
                    Flag = false;
                    _LgOvlpFlag.push_back(Flag);
                    long_ovlp_list.push_back(0);
                }
            }
            else{
                LongOvlp = -(long_threshold_ovlp[i] + (_sv_point_list[i].s_rel + _surdVeh_Length/2 + VEHCALPRMFORREARAXISTOREARBUMPER));
                if (LongOvlp < 0){
                    Flag = true;
                    _LgOvlpFlag.push_back(Flag);
                    long_ovlp_list.push_back(LongOvlp);
                }
                else{
                    Flag = false;
                    _LgOvlpFlag.push_back(Flag);
                    long_ovlp_list.push_back(0);
                }
            }
        }
        else{
            LongOvlp = 0.0f;   //表示纵向重叠度为0
            Flag = false;
            _LgOvlpFlag.push_back(Flag);
            long_ovlp_list.push_back(0);
        }
    }
    return long_ovlp_list;
}

std::vector<float> DynamicObjRisk::CalculateLatOvlpDeg(){
    // TODO
    int size_TrjPnt = 0;
    if (!(_egoVeh_TrjPnt.size() == _sv_point_list.size())){
        // std::cout << "ATTENTION: _egoVeh_TrjPnt and _sv_point_list's size is not equal!" << std::endl;
        size_TrjPnt = std::min(_egoVeh_TrjPnt.size(), _sv_point_list.size());
    }
    else{
        size_TrjPnt = _egoVeh_TrjPnt.size();
    }
    float LatOvlp = 0;
    std::vector<float> lat_ovlp_list;
    
    for (int i = 0; i < size_TrjPnt; i++){
        if (_LgOvlpFlag[i]){
            LatOvlp = _latthreshold_list[i] - (abs(_sv_point_list[i].l - _egoVeh_TrjPnt[i].l) - VEHCALPRMVEHWIDTH/2 - _surdVeh_Width/2);
        }
        else{
            LatOvlp = -1;   //表示横向重叠度为0
        }
        if (LatOvlp > 0){
            lat_ovlp_list.push_back(LatOvlp);
        }
        else{
            lat_ovlp_list.push_back(0);
        }
    }
    return lat_ovlp_list;
}

std::vector<float> DynamicObjRisk::CalculateLongMinDis(){
    // TODO
    int size_TrjPnt = 0;
    if (!(_egoVeh_TrjPnt.size() == _sv_point_list.size())){
        size_TrjPnt = std::min(_egoVeh_TrjPnt.size(), _sv_point_list.size());
        // float void_list = 999;
        // return void_list;
    }
    else{
        size_TrjPnt = _egoVeh_TrjPnt.size();
    }
    _LgMinDisFlag.clear();
    std::vector<float> long_dis_list;
    bool Flag = false;
    for (int i = 0; i < size_TrjPnt; i++){
        // if ((abs(_sv_point_list[i].l - _egoVeh_TrjPnt[i].l) - VEHCALPRMVEHWIDTH/2 - _surdVeh_Width/2) < _latthreshold_list[i]){
        if((!_threshold_boundary_flag.empty()) && (_threshold_boundary_flag[i])){
            Flag = true;
            _LgMinDisFlag.push_back(Flag);
            if (_sv_point_list[i].s_rel >= 0){
                float long_dis = _sv_point_list[i].s_rel - VEHCALPRMFORAXLEDSTRETOVEHFRNT - _surdVeh_Length/2;
                if (long_dis > 0){
                    long_dis_list.push_back(long_dis);
                }
                else{
                    long_dis_list.push_back(0);
                }
            }
            else{
                float long_dis = _sv_point_list[i].s_rel + _surdVeh_Length/2 + VEHCALPRMFORREARAXISTOREARBUMPER;
                if (long_dis < 0){
                    long_dis_list.push_back(long_dis);
                }
                else{
                    long_dis_list.push_back(0);
                }
            }
        }
        else{
            Flag = false;
            _LgMinDisFlag.push_back(Flag);
            if (_sv_point_list[i].s_rel >= 0){
                long_dis_list.push_back(PDCOSTDEFAULTMAXPLUS); 
            }
            else{
                long_dis_list.push_back(PDCOSTDEFAULTMAXNGT);
            }
            // long_dis_list.push_back(999);
            // Flag = false;
            // _LgMinDisFlag.push_back(Flag);
        }
    }
    return long_dis_list;
}

std::vector<float> DynamicObjRisk::CalculateLatMinDis(){
    // TODO
    int size_TrjPnt = 0;
    if (!(_egoVeh_TrjPnt.size() == _sv_point_list.size())){
        size_TrjPnt = std::min(_egoVeh_TrjPnt.size(), _sv_point_list.size());
        // float void_list = 999;
        // return void_list;
    }
    else{
        size_TrjPnt = _egoVeh_TrjPnt.size();
    }
    std::vector<float> lat_dis_list;
    for (int i = 0; i < size_TrjPnt; i++){
        if (_LgMinDisFlag[i]){
            float lat_dis = abs(_sv_point_list[i].l - _egoVeh_TrjPnt[i].l) - VEHCALPRMVEHWIDTH/2 - _surdVeh_Width/2;
            if (lat_dis >= 0){
                lat_dis_list.push_back(lat_dis);
            }
            else{
                lat_dis_list.push_back(0);
            }    
        }
        else{
            lat_dis_list.push_back(PDCOSTDEFAULTMAXPLUS);
        }
    }
    return lat_dis_list;
}

float DynamicObjRisk::CalculateLongCrsSfZnItg(std::vector<float> &LongCrsSfZn_list){

    float LongCrsSfZnItg = 0;
    // std::vector<float> LongCrsSfZn_list;
    // LongCrsSfZn_list = CalculateLongOvlpDeg();
    for (int i = 0; i < LongCrsSfZn_list.size(); i++){
        LongCrsSfZnItg += LongCrsSfZn_list[i] * _timesampl;
    }
    return LongCrsSfZnItg;
}

float DynamicObjRisk::CalculateLatCrsSfZnItg(std::vector<float> &LatCrsSfZn_list){

    float LatCrsSfZnItg = 0;
    // std::vector<float> LatCrsSfZn_list;
    // LatCrsSfZn_list = CalculateLatOvlpDeg();
    for (int i = 0; i < LatCrsSfZn_list.size(); i++){
        LatCrsSfZnItg += LatCrsSfZn_list[i] * _timesampl;
    }
    return LatCrsSfZnItg;
}

void DynamicObjRisk::ObserveVal(){
    PDEvaVar_ pdEvaVar_;
    std::vector<float> abs_ttc;
    std::vector<float> abs_td;
    for(auto &ele : m_DynObjRsk_list){
        pdEvaVar_.val_obj_id = ele.ObjID;
        for(auto &ele1 : ele.m_ObjLongPreClsnTTC){
            abs_ttc.push_back(abs(ele1));
        }
        float min_ttc = *min_element(abs_ttc.begin(), abs_ttc.end());
        if(min_ttc <= 2){
             pdEvaVar_.val_ttc = 1;
        }
        else if(min_ttc > 2 && min_ttc <= 4){
            pdEvaVar_.val_ttc = 2;
        }

        for(auto &ele1 : ele.m_ObjLongPreClsnTD){
            abs_td.push_back(abs(ele1));
        }
        float min_td = *min_element(abs_td.begin(), abs_td.end());
        if(min_td <= 0.6){
            pdEvaVar_.val_td = 1;
        }
        else if(min_td > 0.6 && min_td < 1.5){
            pdEvaVar_.val_td = 2;
        }

        pdEvaVar_.val_lat_dist = ele.m_ObjLatMinDis;
        pdEvaVar_.val_lgt_dist = ele.m_ObjLongMinDis;

        pd_eva_val_.push_back(pdEvaVar_);
    }
}


#if FLAG_OUTPUT != 0
void DynamicObjRisk::OutPutToOfs(){

    #if FLAG_DLOG_IOOUT != 0
    DLOG(INFO) << "_tmpdynamicrisk_end is: " << _tmpdynamicrisk.size() << std::endl; 
    for(auto &ele : _tmpdynamicrisk){
        DLOG(INFO) << ele.second.tmp_ObjID << ", ";
    }
    DLOG(INFO) << "ObjInEgoLaneList is: " << ObjInEgoLaneList.size() << std::endl;
    for(auto &ele : ObjInEgoLaneList){
        DLOG(INFO) << ele.object_id << ", ";
    }
    DLOG(INFO) << "ObjInEgoLaneListL is: " << ObjInEgoLaneListL.size() << std::endl;
    for(auto &ele : ObjInEgoLaneListL){
        DLOG(INFO) << ele.object_id << ", ";
    }
    DLOG(INFO) << "ObjInEgoLaneListR is: " << ObjInEgoLaneListR.size() << std::endl;
    for(auto &ele : ObjInEgoLaneListR){
        DLOG(INFO) << ele.object_id << ", ";
    }
    DLOG(INFO) << "ObjInEgoLaneListLL is: " << ObjInEgoLaneListLL.size() << std::endl;
    for(auto &ele : ObjInEgoLaneListLL){
        DLOG(INFO) << ele.object_id << ", ";
    }
    DLOG(INFO) << "ObjInEgoLaneListRR is: " << ObjInEgoLaneListRR.size() << std::endl;
    for(auto &ele : ObjInEgoLaneListRR){
        DLOG(INFO) << ele.object_id << ", ";
    }
    DLOG(INFO) << "ObjListOther is: " << ObjListOther.size() << std::endl;
    for(auto &ele : ObjListOther){
        DLOG(INFO) << ele.object_id << ", ";
    }

    std::ofstream ofs;
    struct timeval time;
    gettimeofday(&time, NULL);
    int size_TrjPnt_ = 0;
    if (!(_egoVeh_TrjPnt.size() == _sv_point_list.size())){
        size_TrjPnt_ = std::min(_egoVeh_TrjPnt.size(), _sv_point_list.size());
    }
    else{
        size_TrjPnt_ = _egoVeh_TrjPnt.size();
    }
    DLOG(INFO) << "start----------------------------------------------------------------------------";
    // ActionType
    DLOG(INFO) << "action_type: " << int(Action_Type) 
               << ", time: " << time.tv_sec << "." << time.tv_usec;

    // EV
    DLOG(INFO) << "<EV>" << ", act: " << int(Action_Type) << ", RoadID:(" << int(RoadID.x) << " " << int(RoadID.y) << ")" 
               << ", PassageID:(" << int(PassageID.x) << " " << int(PassageID.y) << ")"
               << ", time: " << time.tv_sec << "." << time.tv_usec;
    for (auto &ele : _egoVeh_TrjPnt){
        DLOG(INFO) << "t: " << ele.t << ",  Es: " << ele.s << ",  El: " << ele.l << ",  Evs: " << ele.vs << ", Eas: " << ele.as;
    }
    DLOG(INFO) << " ";

    // SV
    DLOG(INFO) << "<SV>" << ", act: " << int(Action_Type) << ", RoadID:(" << int(RoadID.x) << " " << int(RoadID.y) << ")" 
               << ", PassageID:(" << int(PassageID.x) << " " << int(PassageID.y) << ")" 
               << ", time: " << time.tv_sec << "." << time.tv_usec;
    int i = 1;
    for (auto &ele : _allSurdVeh_TrjPnt){
        if (ele.obj_ID == 131){
        // if (ele.obj_ID == 133 || ele.obj_ID == 144 || ele.obj_ID == 2300 || ele.obj_ID == 5611){
        DLOG(INFO) << "Act: " << int(Action_Type) << ", obj_ID: " << ele.obj_ID <<", " << i << " width: " << ele.surdVeh_Width << " length: " << ele.surdVeh_Length;
        for(int i = 0; i < size_TrjPnt_; i++){
            DLOG(INFO) << "t: " << ele.sv_point_list[i].t << ",  s_rel: " << ele.sv_point_list[i].s_rel 
                       << ",  l_rel: " << (abs(ele.sv_point_list[i].l - _egoVeh_TrjPnt[i].l) - VEHCALPRMVEHWIDTH/2 - _surdVeh_Width/2) << ", l: " << ele.sv_point_list[i].l
                       << ",  vs: " << ele.sv_point_list[i].vs << ",  vl: " << ele.sv_point_list[i].vl;
        }
        i++;
        }
        DLOG(INFO) << " ";
    }

    // Risk
    DLOG(INFO) << "<Risk>" << ", act: " << int(Action_Type) << ", RoadID:(" << int(RoadID.x) << " " << int(RoadID.y) << ")" 
               << ", PassageID:(" << int(PassageID.x) << " " << int(PassageID.y) << ")" 
               << ", time: " << time.tv_sec << "." << time.tv_usec;
    for (int i = 0; i < m_DynObjRsk_list.size(); i++){
        if (m_DynObjRsk_list[i].ObjID == 131){
        // if (m_DynObjRsk_list[i].ObjID == 133 || m_DynObjRsk_list[i].ObjID == 144 || m_DynObjRsk_list[i].ObjID == 2300 || m_DynObjRsk_list[i].ObjID == 5611){
        DLOG(INFO) << "Act: " << int(Action_Type) << ", ObjID: " << m_DynObjRsk_list[i].ObjID << ", " << i+1;
        for (int j = 0; j < m_DynObjRsk_list[i].m_ObjLongPreClsnTTC.size(); j++){
            if (j == 0){
                DLOG(INFO) << "t:  " << j*0.1
                           << ", ttc: " << m_DynObjRsk_list[i].m_ObjLongPreClsnTTC[j]
                           << ", td: " << m_DynObjRsk_list[i].m_ObjLongPreClsnTD[j]
                           << ", LgOvlp: " << m_DynObjRsk_list[i].m_ObjLongOvlpDeg[j]
                           << ", LatOvlp: " << m_DynObjRsk_list[i].m_ObjLatOvlpDeg[j]
                           << ", LgMinDis: " << m_DynObjRsk_list[i].m_ObjLongMinDis
                           << ", LatMinDis: " << m_DynObjRsk_list[i].m_ObjLatMinDis
                           << ", LgZnItg: " << m_DynObjRsk_list[i].m_ObjLongCrsSfZnItg
                           << ", LatZnItg: " << m_DynObjRsk_list[i].m_ObjLatCrsSfZnItg;
            }
            else{
                DLOG(INFO) << "t:  " << j*0.1
                           << ", ttc: " << m_DynObjRsk_list[i].m_ObjLongPreClsnTTC[j]
                           << ", td: " << m_DynObjRsk_list[i].m_ObjLongPreClsnTD[j]
                           << ", LgOvlp: " << m_DynObjRsk_list[i].m_ObjLongOvlpDeg[j]
                           << ", LatOvlp: " << m_DynObjRsk_list[i].m_ObjLatOvlpDeg[j];
            }
        }
        }
        DLOG(INFO) << " ";
    }
    DLOG(INFO) << "end------------------------------------------------------------------------------";
    #endif


    #if FLAG_OFS_IOOUT != 0
    std::ofstream ofs;
    struct timeval time;
    gettimeofday(&time, NULL);
    int size_TrjPnt_ = 0;
    if (!(_egoVeh_TrjPnt.size() == _sv_point_list.size())){
        // std::cout << "ATTENTION: _egoVeh_TrjPnt and _sv_point_list's size is not equal!" << std::endl;
        size_TrjPnt_ = std::min(_egoVeh_TrjPnt.size(), _sv_point_list.size());
    }
    else{
        size_TrjPnt_ = _egoVeh_TrjPnt.size();
    }
    // ActionType
    ofs.open("./data_log/LogFile/Draw/Draw_Act.txt", std::ios::app);
    ofs << "action_type: " << int(Action_Type) 
        << ", time: " << time.tv_sec << "." << time.tv_usec << std::endl;
    ofs.close();

    // EV
    // ofs.open("./src/mpc_drv_ctAp_ba/core/source/PD_module/LogFile/Draw/Draw_EV.txt", std::ios::app);
    ofs.open("./data_log/LogFile/Draw/Draw_EV.txt", std::ios::app);
    ofs << "EV" << ", time," << time.tv_sec << "." << time.tv_usec << ", act," << int(Action_Type) 
        << ", RoadID:(" << int(RoadID.x) << " " << int(RoadID.y) << ")" 
        << ", PassageID:(" << int(PassageID.x) << " " << int(PassageID.y) << ")" << std::endl;
    for (auto &ele : _egoVeh_TrjPnt){
        ofs << "t," << ele.t << ",  Es," << ele.s << ",  El," << ele.l 
            << ",  Evs," << ele.vs << std::endl;
    }
    ofs << std::endl;
    // ofs << "------------------------------------------------------------------------" << std::endl;
    ofs.close();

    // SV
    // ofs.open("./src/mpc_drv_ctAp_ba/core/source/PD_module/LogFile/Draw/Draw_SV.txt", std::ios::app);
    ofs.open("./data_log/LogFile/Draw/Draw_SV.txt", std::ios::app);
    ofs << "act," << int(Action_Type) << ", RoadID:(" << int(RoadID.x) << " " << int(RoadID.y) << ")" 
        << ", PassageID:(" << int(PassageID.x) << " " << int(PassageID.y) << ")" 
        << ", time," << time.tv_sec << "." << time.tv_usec<< std::endl;
    int i = 1;
    for (auto &ele : _allSurdVeh_TrjPnt){
        // if (ele.obj_ID == 88){
        // if ((ele.obj_ID == 198) || (ele.obj_ID == 98)){
        ofs << "act," << int(Action_Type) << " obj_ID," << ele.obj_ID <<", " << i << " width," << ele.surdVeh_Width << " length," << ele.surdVeh_Length << std::endl;
        for(int i = 0; i < size_TrjPnt_; i++){
            // ofs << "t," << ele.sv_point_list[i].t << ",  s_rel," << ele.sv_point_list[i].s_rel << ", thred," << _latthreshold_list[i] << ", thred_flag," << _threshold_boundary_flag[i]
            //     << ",  l_rel," << (abs(ele.sv_point_list[i].l - _egoVeh_TrjPnt[i].l) - VEHCALPRMVEHWIDTH/2 - _surdVeh_Width/2) << ", l," << ele.sv_point_list[i].l
            //     << ",  vs," << ele.sv_point_list[i].vs << ",  vl," << ele.sv_point_list[i].vl << std::endl;
            ofs << "t," << ele.sv_point_list[i].t << ",  s_rel," << ele.sv_point_list[i].s_rel 
                << ",  l_rel," << (abs(ele.sv_point_list[i].l - _egoVeh_TrjPnt[i].l) - VEHCALPRMVEHWIDTH/2 - _surdVeh_Width/2) << ", l," << ele.sv_point_list[i].l
                << ",  vs," << ele.sv_point_list[i].vs << ",  vl," << ele.sv_point_list[i].vl << std::endl;
        }
        ofs << std::endl;
        i++;
        // }
    }
    // ofs << std::endl;
    // ofs << "------------------------------------------------------------------------" << std::endl;
    ofs.close();

    // Risk
    // ofs.open("./src/mpc_drv_ctAp_ba/core/source/PD_module/LogFile/Draw/Draw_Risk.txt", std::ios::app);
    ofs.open("./data_log/LogFile/Draw/Draw_Risk.txt", std::ios::app);
    ofs << "act," << int(Action_Type) << ", RoadID:(" << int(RoadID.x) << " " << int(RoadID.y) << ")" 
        << ", PassageID:(" << int(PassageID.x) << " " << int(PassageID.y) << ")" 
        << ", time," << time.tv_sec << "." << time.tv_usec<< std::endl;
    for (int i = 0; i < m_DynObjRsk_list.size(); i++){
        // if (m_DynObjRsk_list[i].ObjID == 88){
        // if ((m_DynObjRsk_list[i].ObjID == 198) || (m_DynObjRsk_list[i].ObjID == 98)){
        ofs << "act," << int(Action_Type) << " ObjID," << m_DynObjRsk_list[i].ObjID << ", " << i+1 << std::endl;
        for (int j = 0; j < m_DynObjRsk_list[i].m_ObjLongPreClsnTTC.size(); j++){
            if (j == 0){
                ofs << "t, " << j*0.1;
                ofs << ", ttc," << m_DynObjRsk_list[i].m_ObjLongPreClsnTTC[j];
                ofs << ", td," << m_DynObjRsk_list[i].m_ObjLongPreClsnTD[j];
                ofs << ", LgOvlp," << m_DynObjRsk_list[i].m_ObjLongOvlpDeg[j];
                ofs << ", LatOvlp," << m_DynObjRsk_list[i].m_ObjLatOvlpDeg[j];
                ofs << ", LgMinDis," << m_DynObjRsk_list[i].m_ObjLongMinDis;
                ofs << ", LatMinDis," << m_DynObjRsk_list[i].m_ObjLatMinDis;
                ofs << ", LgZnItg," << m_DynObjRsk_list[i].m_ObjLongCrsSfZnItg;
                ofs << ", LatZnItg," << m_DynObjRsk_list[i].m_ObjLatCrsSfZnItg;
                ofs << std::endl;
            }
            else{
                ofs << "t, " << j*0.1;
                ofs << ", ttc," << m_DynObjRsk_list[i].m_ObjLongPreClsnTTC[j];
                ofs << ", td," << m_DynObjRsk_list[i].m_ObjLongPreClsnTD[j];
                ofs << ", LgOvlp," << m_DynObjRsk_list[i].m_ObjLongOvlpDeg[j];
                ofs << ", LatOvlp," << m_DynObjRsk_list[i].m_ObjLatOvlpDeg[j];
                ofs << std::endl;
            }
        }
        ofs << std::endl;
        // }
    }
    // ofs << std::endl;
    // ofs << "------------------------------------------------------------------------" << std::endl;
    ofs.close();
    #endif
}
#endif

